local enet = require('enet')
local entity = require('entity')

--Choose either variable depending on if we start a server or the client hosts a server
local clientHost
local serverHost

local peers = {} --This keeps track of all the connected clients
local port = 7777 --Port we are using
local receivedData = false

local server = {} --We will return this value


-- Callbacks to invoke when certain events are received from a peer define a callback to handle every type of message in our application protocol
local message_handlers = {
	['your-id'] = function(message, event, is_server)
		local player_id = message[2]
		local x_pos = message[3]
		local y_pos = message[4]
		entity.id = player_id
		entity.entities[player_id] = entity.spawn(player_id, x_pos, y_pos)
	end,
	['peer-id'] = function(message, event, is_server)
		local player_id = message[2]
		local x_pos = message[3]
		local y_pos = message[4]
		entity.entities[player_id] = entity.spawn(player_id, x_pos, y_pos)
	end,
	['move'] = function(message, event, is_server)
		local player_id = message[2]
		local x_pos = message[3]
		local y_pos = message[4]
		entity.entities[player_id].x = x_pos
		entity.entities[player_id].y = y_pos
		if is_server then
		-- Relay this message to the other players
			for id, peer in pairs(peers) do
				if id ~= player_id then
					peer:send(event.data)
				end
			end
		end
	end
}

-- These event types are defined by Lua-enet. A "receive" type of event is a generic event that carries any of the messages above.
local event_handlers = {

	connect = function(event, is_server)
		--Only server needs to do stuff during connect
		if is_server then
			local player = entity.spawn(tostring(event.peer:connect_id()), 100, 100) --connect_id() creates a number as an id
			entity.entities[player.id] = player --Store this player in the player table with the player ID as the key.
			event.peer:send('your-id|' .. player.id .. '|' .. player.x .. '|' .. player.y) -- Send the initial "your-id" message back to the connecting client so they can spawn this entity too.

			-- Let all the other peers know about this player
			for _, peer in pairs(peers) do
				local peer_player = entity.entities[tostring(peer:connect_id())]
				peer:send('peer-id|' .. player.id .. '|' .. player.x .. '|' .. player.y)
				event.peer:send('peer-id|' .. peer_player.id .. '|' .. player.x .. '|' .. player.y)
			end
			peers[tostring(event.peer:connect_id())] = event.peer --Add this peer to the peer list
		end
	end,

	disconnect = function(event, is_server)
	end,

	receive = function(event, is_server)
		local message = {}
		for match in (event.data .. '|'):gmatch('(.-)|') do
			table.insert(message, match)
		end
		message_handlers[message[1]](message, event, is_server)
	end
}


server.startServer = function() --We will start the server
	serverHost = enet.host_create('localhost:'..port)
	serverStarted = true
end

server.startClient = function()
	clientHost = enet.host_create()
	serverHost = clientHost:connect('localhost:'..port)
end

server.isConnected = function()
	return receivedData
end

server.isClient = function()
	return clientHost and true or false
end

server.isServer = function()
	return serverHost and not clientHost
end

server.update = function()
	local host = clientHost or serverHost
	if not host then return end --Check if host is defined
	local event = host:service() --Wait for any packets (messages)
	if event then --If data then continue
		receivedData = true --server.isConnected = true
		event_handlers[event.type](event, server.isServer()) --event.type will be either "connect", "disconnect", or "receive"
		--print out table for debug
	  	print('----')
		for k, v in pairs(event) do
			print(k, v)
		end
	end
end
  
server.disconnect = function()
	for _, peer in ipairs(peers) do
		peer:disconnect_now()
	end
	peers = {}

	if server.isClient() then
		serverHost:disconnect_now()
		clientHost = nil
	end

	serverHost = nil
	receivedData = false
	serverStarted = false
end

server.send = function(message)
	if server.isClient() then
		serverHost:send(message)
	else
		serverHost:broadcast(message)
	end
end

return server